import { useContext, useEffect, useState } from 'react';
import { getInfos } from './api';
import { BCD_URL_PREFIX, CONTRACT_ADDRESS, NETWORK_TYPE, FETCH_INTERVAL } from './constant';
import { Context } from './context/context';


function App() {

  const [infos, setInfos] = useState()
  const [total, setTotal] = useState()

  const [holderAddress, setHolderAddress] = useState("")
  const [holderShare, setHolderShare] = useState("")

  const context = useContext(Context)

  const getAPIdata = () => {
    getInfos().then((res) => {
      setInfos(res.infos)
      setTotal(res.total)
    })
  }

  useEffect(() => {
    getAPIdata()
    const timerId = setInterval(getAPIdata, FETCH_INTERVAL)
    return () => clearInterval(timerId)
  }, [])

  const handleSyncUnsync = async () => {
    if (context.address === undefined) { context.syncTaquito() }
    else { context.disconnect() }
  }

  const sendTezos = (amount) => {
    if(!context.acc || !context.acc.address){
      alert("Please connect wallet")
      return
    }
    context.send(amount).then(() => {
      alert("Transaction applied!")
    })
  }

  const updateShare = () => {
    if(!context.acc || !context.acc.address){
      alert("Please connect wallet")
      return
    }
    context.updateSingleShare(holderAddress, holderShare).then(() => {
      alert("Transaction applied!")
    })
  }

  return (
    <div className='p-10 text-white'>
      <div className='p-10 text-3xl'> - Tezos smart contract and web dapps sample - </div>
      <div>

        <button
          className='px-6 py-4 bg-sky-600 rounded-md'
          onClick={handleSyncUnsync}
        >
          {context.acc && context.acc.address ? context.acc.address : "CONNECT WALLET"}
        </button>

        <div className='mt-6 flex flex-row justify-around items-center'>
          <a href={`${BCD_URL_PREFIX}/${NETWORK_TYPE}/${CONTRACT_ADDRESS}`} className='text-left' target="_blank">{CONTRACT_ADDRESS}</a>
          <button
            className='px-6 py-2 bg-sky-600 rounded-md'
            onClick={() => sendTezos(100)}
          >
            Send 100 xtz
          </button>
        </div>

      </div>
      <div className='text-left border-t mt-2 pt-2'>
        <div className='text-xl'>[ data from SC ]</div>
        <div className='text-center'>
          <table className='w-full'>
            <thead>
              <tr>
                <th>Wallet Address</th>
                <th>Shares</th>
                <th>Balances</th>
              </tr>
            </thead>
            <tbody>
              {
                infos && infos.length > 0 && infos.map((s, i) =>
                  <tr key={i} className="border w-full">
                    <td>{s.address}</td>
                    <td>{s.share}</td>
                    <td>{s.balance / 1000000} tez</td>
                  </tr>
                )
              }
            </tbody>
          </table>
          <p className='py-2'>current total shares: {total}</p>
        </div>
      </div>
      <div className='text-left mt-2 pt-2'>
        <div className='text-xl'>[ Interact with SC ]</div>
        <div className='text-center'>
          <label>Wallet address</label>
          <input
            className='mx-4 p-2 w-80 rounded-md text-black'
            onChange={(e) => setHolderAddress(e.target.value)}
          />
          <label>Shares</label>
          <input
            className='ml-4 p-2 w-20 rounded-md text-black'
            onChange={(e) => setHolderShare(e.target.value)}
          ></input>
          <button
            className='ml-4 px-6 py-2 bg-sky-600 rounded-md'
            onClick={() => updateShare()}
          >
            Update Share
          </button>
        </div>
      </div>
    </div>
  );
}

export default App;
