import React, { createContext, Component } from 'react'
import { BeaconWallet } from '@taquito/beacon-wallet'
import { CONTRACT_ADDRESS, NETWORK_TYPE, NODE_URL } from 'src/constant'
import { TezosToolkit } from '@taquito/taquito'

export const Context = createContext()

const Tezos = new TezosToolkit(NODE_URL)
const wallet = new BeaconWallet({
	name: 'dapps2023-sample'
})
Tezos.setWalletProvider(wallet)

export default class ContextProvider extends Component {
	constructor(props) {
		super(props)

		this.state = {

			/* taquito */
			Tezos: null,
			wallet: null,
			acc: null,

			updateMessage: (message) => this.setState({ message: message }),

			setAccount: async () => {
				this.setState({
					acc:
						Tezos !== undefined
							? await wallet.client.getActiveAccount()
							: undefined,
					address: await wallet.client.getActiveAccount(),
				})
			},

			syncTaquito: async () => {
				const network = {
					type: "custom",
					rpcUrl: NODE_URL,
				}

				// We check the storage and only do a permission request if we don't have an active account yet
				// This piece of code should be called on startup to "load" the current address from the user
				// If the activeAccount is present, no "permission request" is required again, unless the user "disconnects" first.
				let activeAccount = await wallet.client.getActiveAccount()
				if (activeAccount === undefined) {
					await wallet.clearActiveAccount()
					await wallet.requestPermissions({ network })
						.then((response) => {
							console.log(response)
						})
						.catch((e) => console.error(e))
					// const result = await this.state.sign(`${AUTH_MESSAGE_PREFIX} ${new Date().toISOString()}`)
					// signIn('beacon', { redirect: false, ...result })
				}
				this.setState({
					Tezos: Tezos,
					address: await wallet.getPKH(),
					acc: await wallet.client.getActiveAccount(),
					wallet,
				})
				// this.state.setAuth(await wallet.getPKH())
			},

			disconnect: async () => {
				console.log('disconnect wallet')
				// This will clear the active account and the next "syncTaquito" will trigger a new sync
				await wallet.client.clearActiveAccount()
				// signOut({ redirect: false })
				this.setState({
					address: undefined,
					acc: undefined,
				})
			},

			updateSingleShare: async (address, share) => {

				const res = await Tezos.wallet
					.at(CONTRACT_ADDRESS)
					.then(async (c) =>
						c.methods
							.update_share([
								{
									addr: address,
									share: parseInt(share)
								}
							])
							.send({ amount: 0 })
							.then(async (op) => {
								console.log("Hash : " + op.opHash)

								return op.confirmation()
									.then((result) => {
										if (result.completed) {
											console.log('Update Share Success!');
											return {
												status: "success",
												ophash: op.opHash
											};
										} else {
											console.log('An error has occurred when updating');
											return {
												status: "failed",
												ophash: op.opHash
											};
										}
									})
							})
					)
					.catch((e) => e)

				return res
			},

			send: async (tezosAmount) => {

				const res = await Tezos.wallet
					.at(CONTRACT_ADDRESS)
					.then(async (c) =>
						c.methods
							.default()
							.send({ amount: parseInt(tezosAmount*1000000), mutez: true })
							.then(async (op) => {
								console.log("Hash : " + op.opHash)

								return op.confirmation()
									.then((result) => {
										if (result.completed) {
											console.log('Send Money Success!');
											return {
												status: "success",
												ophash: op.opHash
											};
										} else {
											console.log('An error has occurred when sending');
											return {
												status: "failed",
												ophash: op.opHash
											};
										}
									})
							})
					)
					.catch((e) => e)

				return res
			}
		}
	}

	render() {
		return (
			<Context.Provider
				value={{
					...this.state,
				}}
			>
				{this.props.children}
			</Context.Provider>
		)
	}
}
